/* $RCSfile: EXTERN.h,v $$Revision: 4.1 $$Date: 92/08/07 18:29:05 $
 *
 *    Copyright (C) 1991, 1992, 1993, 1994, by Larry Wall and others
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 * $Log:	EXTERN.h,v $
 */

#undef EXT
#define EXT extern

#undef INIT
#define INIT(x)

#undef DOINIT
