# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

return <<'END';
0041	005A	LATIN
0061	007A	LATIN
00AA		LATIN
00B5		GREEK
00BA		LATIN
00C0	00D6	LATIN
00D8	00F6	LATIN
00F8	0236	LATIN
0250	02B8	LATIN
02E0	02E4	LATIN
0300	034F	INHERITED
0360	036F	INHERITED
037A		GREEK
0386		GREEK
0388	038A	GREEK
038C		GREEK
038E	03A1	GREEK
03A3	03CE	GREEK
03D0	03F5	GREEK
03F7	03FB	GREEK
0400	0481	CYRILLIC
0483	0486	CYRILLIC
0488	0489	INHERITED
048A	04CE	CYRILLIC
04D0	04F5	CYRILLIC
04F8	04F9	CYRILLIC
0500	050F	CYRILLIC
0531	0556	ARMENIAN
0559		ARMENIAN
0561	0587	ARMENIAN
0591	05A1	INHERITED
05A3	05B9	INHERITED
05BB	05BD	INHERITED
05BF		INHERITED
05C1	05C2	INHERITED
05C4		INHERITED
05D0	05EA	HEBREW
05F0	05F2	HEBREW
0621	063A	ARABIC
0641	064A	ARABIC
064B	0655	INHERITED
066E	066F	ARABIC
0670		INHERITED
0671	06D3	ARABIC
06D5		ARABIC
06D6	06E4	INHERITED
06E5	06E6	ARABIC
06E7	06E8	INHERITED
06EA	06ED	INHERITED
06EE	06EF	ARABIC
06FA	06FC	ARABIC
06FF		ARABIC
0710	074A	SYRIAC
074D	074F	SYRIAC
0780	07B1	THAANA
0901	0939	DEVANAGARI
093C	094D	DEVANAGARI
0950	0954	DEVANAGARI
0958	0963	DEVANAGARI
0966	096F	DEVANAGARI
0981	0983	BENGALI
0985	098C	BENGALI
098F	0990	BENGALI
0993	09A8	BENGALI
09AA	09B0	BENGALI
09B2		BENGALI
09B6	09B9	BENGALI
09BC	09C4	BENGALI
09C7	09C8	BENGALI
09CB	09CD	BENGALI
09D7		BENGALI
09DC	09DD	BENGALI
09DF	09E3	BENGALI
09E6	09F1	BENGALI
0A02	0A03	GURMUKHI
0A05	0A0A	GURMUKHI
0A0F	0A10	GURMUKHI
0A13	0A28	GURMUKHI
0A2A	0A30	GURMUKHI
0A32	0A33	GURMUKHI
0A35	0A36	GURMUKHI
0A38	0A39	GURMUKHI
0A3C		GURMUKHI
0A3E	0A42	GURMUKHI
0A47	0A48	GURMUKHI
0A4B	0A4D	GURMUKHI
0A59	0A5C	GURMUKHI
0A5E		GURMUKHI
0A66	0A74	GURMUKHI
0A81	0A83	GUJARATI
0A85	0A8D	GUJARATI
0A8F	0A91	GUJARATI
0A93	0AA8	GUJARATI
0AAA	0AB0	GUJARATI
0AB2	0AB3	GUJARATI
0AB5	0AB9	GUJARATI
0ABC	0AC5	GUJARATI
0AC7	0AC9	GUJARATI
0ACB	0ACD	GUJARATI
0AD0		GUJARATI
0AE0	0AE3	GUJARATI
0AE6	0AEF	GUJARATI
0B01	0B03	ORIYA
0B05	0B0C	ORIYA
0B0F	0B10	ORIYA
0B13	0B28	ORIYA
0B2A	0B30	ORIYA
0B32	0B33	ORIYA
0B35	0B39	ORIYA
0B3C	0B43	ORIYA
0B47	0B48	ORIYA
0B4B	0B4D	ORIYA
0B56	0B57	ORIYA
0B5C	0B5D	ORIYA
0B5F	0B61	ORIYA
0B66	0B6F	ORIYA
0B71		ORIYA
0B82	0B83	TAMIL
0B85	0B8A	TAMIL
0B8E	0B90	TAMIL
0B92	0B95	TAMIL
0B99	0B9A	TAMIL
0B9C		TAMIL
0B9E	0B9F	TAMIL
0BA3	0BA4	TAMIL
0BA8	0BAA	TAMIL
0BAE	0BB5	TAMIL
0BB7	0BB9	TAMIL
0BBE	0BC2	TAMIL
0BC6	0BC8	TAMIL
0BCA	0BCD	TAMIL
0BD7		TAMIL
0BE7	0BF2	TAMIL
0C01	0C03	TELUGU
0C05	0C0C	TELUGU
0C0E	0C10	TELUGU
0C12	0C28	TELUGU
0C2A	0C33	TELUGU
0C35	0C39	TELUGU
0C3E	0C44	TELUGU
0C46	0C48	TELUGU
0C4A	0C4D	TELUGU
0C55	0C56	TELUGU
0C60	0C61	TELUGU
0C66	0C6F	TELUGU
0C82	0C83	KANNADA
0C85	0C8C	KANNADA
0C8E	0C90	KANNADA
0C92	0CA8	KANNADA
0CAA	0CB3	KANNADA
0CB5	0CB9	KANNADA
0CBD	0CC4	KANNADA
0CC6	0CC8	KANNADA
0CCA	0CCD	KANNADA
0CD5	0CD6	KANNADA
0CDE		KANNADA
0CE0	0CE1	KANNADA
0CE6	0CEF	KANNADA
0D02	0D03	MALAYALAM
0D05	0D0C	MALAYALAM
0D0E	0D10	MALAYALAM
0D12	0D28	MALAYALAM
0D2A	0D39	MALAYALAM
0D3E	0D43	MALAYALAM
0D46	0D48	MALAYALAM
0D4A	0D4D	MALAYALAM
0D57		MALAYALAM
0D60	0D61	MALAYALAM
0D66	0D6F	MALAYALAM
0D82	0D83	SINHALA
0D85	0D96	SINHALA
0D9A	0DB1	SINHALA
0DB3	0DBB	SINHALA
0DBD		SINHALA
0DC0	0DC6	SINHALA
0DCA		SINHALA
0DCF	0DD4	SINHALA
0DD6		SINHALA
0DD8	0DDF	SINHALA
0DF2	0DF3	SINHALA
0E01	0E3A	THAI
0E40	0E4E	THAI
0E50	0E59	THAI
0E81	0E82	LAO
0E84		LAO
0E87	0E88	LAO
0E8A		LAO
0E8D		LAO
0E94	0E97	LAO
0E99	0E9F	LAO
0EA1	0EA3	LAO
0EA5		LAO
0EA7		LAO
0EAA	0EAB	LAO
0EAD	0EB9	LAO
0EBB	0EBD	LAO
0EC0	0EC4	LAO
0EC6		LAO
0EC8	0ECD	LAO
0ED0	0ED9	LAO
0EDC	0EDD	LAO
0F00		TIBETAN
0F18	0F19	TIBETAN
0F20	0F33	TIBETAN
0F35		TIBETAN
0F37		TIBETAN
0F39		TIBETAN
0F40	0F47	TIBETAN
0F49	0F6A	TIBETAN
0F71	0F84	TIBETAN
0F86	0F8B	TIBETAN
0F90	0F97	TIBETAN
0F99	0FBC	TIBETAN
0FC6		TIBETAN
1000	1021	MYANMAR
1023	1027	MYANMAR
1029	102A	MYANMAR
102C	1032	MYANMAR
1036	1039	MYANMAR
1040	1049	MYANMAR
1050	1059	MYANMAR
10A0	10C5	GEORGIAN
10D0	10F8	GEORGIAN
1100	1159	HANGUL
115F	11A2	HANGUL
11A8	11F9	HANGUL
1200	1206	ETHIOPIC
1208	1246	ETHIOPIC
1248		ETHIOPIC
124A	124D	ETHIOPIC
1250	1256	ETHIOPIC
1258		ETHIOPIC
125A	125D	ETHIOPIC
1260	1286	ETHIOPIC
1288		ETHIOPIC
128A	128D	ETHIOPIC
1290	12AE	ETHIOPIC
12B0		ETHIOPIC
12B2	12B5	ETHIOPIC
12B8	12BE	ETHIOPIC
12C0		ETHIOPIC
12C2	12C5	ETHIOPIC
12C8	12CE	ETHIOPIC
12D0	12D6	ETHIOPIC
12D8	12EE	ETHIOPIC
12F0	130E	ETHIOPIC
1310		ETHIOPIC
1312	1315	ETHIOPIC
1318	131E	ETHIOPIC
1320	1346	ETHIOPIC
1348	135A	ETHIOPIC
1369	137C	ETHIOPIC
13A0	13F4	CHEROKEE
1401	166C	CANADIAN_ABORIGINAL
166F	1676	CANADIAN_ABORIGINAL
1681	169A	OGHAM
16A0	16EA	RUNIC
16EE	16F0	RUNIC
1700	170C	TAGALOG
170E	1714	TAGALOG
1720	1734	HANUNOO
1740	1753	BUHID
1760	176C	TAGBANWA
176E	1770	TAGBANWA
1772	1773	TAGBANWA
1780	17D3	KHMER
17E0	17E9	KHMER
180B	180D	INHERITED
1810	1819	MONGOLIAN
1820	1877	MONGOLIAN
1880	18A9	MONGOLIAN
1900	191C	LIMBU
1920	192B	LIMBU
1930	193B	LIMBU
1946	194F	LIMBU
1950	196D	TAI_LE
1970	1974	TAI_LE
1D00	1D25	LATIN
1D26	1D2A	GREEK
1D2B		CYRILLIC
1D2C	1D5C	LATIN
1D5D	1D61	GREEK
1D62	1D65	LATIN
1D66	1D6A	GREEK
1D6B		LATIN
1E00	1E9B	LATIN
1EA0	1EF9	LATIN
1F00	1F15	GREEK
1F18	1F1D	GREEK
1F20	1F45	GREEK
1F48	1F4D	GREEK
1F50	1F57	GREEK
1F59		GREEK
1F5B		GREEK
1F5D		GREEK
1F5F	1F7D	GREEK
1F80	1FB4	GREEK
1FB6	1FBC	GREEK
1FBE		GREEK
1FC2	1FC4	GREEK
1FC6	1FCC	GREEK
1FD0	1FD3	GREEK
1FD6	1FDB	GREEK
1FE0	1FEC	GREEK
1FF2	1FF4	GREEK
1FF6	1FFC	GREEK
2071		LATIN
207F		LATIN
20D0	20EA	INHERITED
2126		GREEK
212A	212B	LATIN
2800	28FF	BRAILLE
2E80	2E99	HAN
2E9B	2EF3	HAN
2F00	2FD5	HAN
3005		HAN
3007		HAN
3021	3029	HAN
302A	302F	INHERITED
3038	303B	HAN
3041	3096	HIRAGANA
3099	309A	INHERITED
309D	309F	HIRAGANA
30A1	30FA	KATAKANA
30FD	30FF	KATAKANA
3105	312C	BOPOMOFO
3131	318E	HANGUL
31A0	31B7	BOPOMOFO
31F0	31FF	KATAKANA
3400	4DB5	HAN
4E00	9FA5	HAN
A000	A48C	YI
A490	A4C6	YI
AC00	D7A3	HANGUL
F900	FA2D	HAN
FA30	FA6A	HAN
FB00	FB06	LATIN
FB13	FB17	ARMENIAN
FB1D		HEBREW
FB1E		INHERITED
FB1F	FB28	HEBREW
FB2A	FB36	HEBREW
FB38	FB3C	HEBREW
FB3E		HEBREW
FB40	FB41	HEBREW
FB43	FB44	HEBREW
FB46	FB4F	HEBREW
FB50	FBB1	ARABIC
FBD3	FD3D	ARABIC
FD50	FD8F	ARABIC
FD92	FDC7	ARABIC
FDF0	FDFB	ARABIC
FE00	FE0F	INHERITED
FE20	FE23	INHERITED
FE70	FE74	ARABIC
FE76	FEFC	ARABIC
FF21	FF3A	LATIN
FF41	FF5A	LATIN
FF66	FF6F	KATAKANA
FF71	FF9D	KATAKANA
FFA0	FFBE	HANGUL
FFC2	FFC7	HANGUL
FFCA	FFCF	HANGUL
FFD2	FFD7	HANGUL
FFDA	FFDC	HANGUL
10000	1000B	LINEAR_B
1000D	10026	LINEAR_B
10028	1003A	LINEAR_B
1003C	1003D	LINEAR_B
1003F	1004D	LINEAR_B
10050	1005D	LINEAR_B
10080	100FA	LINEAR_B
10300	1031E	OLD_ITALIC
10330	1034A	GOTHIC
10380	1039D	UGARITIC
10400	1044F	DESERET
10450	1047F	SHAVIAN
10480	1049D	OSMANYA
104A0	104A9	OSMANYA
10800	10805	CYPRIOT
10808		CYPRIOT
1080A	10835	CYPRIOT
10837	10838	CYPRIOT
1083C		CYPRIOT
1083F		CYPRIOT
1D167	1D169	INHERITED
1D17B	1D182	INHERITED
1D185	1D18B	INHERITED
1D1AA	1D1AD	INHERITED
20000	2A6D6	HAN
2F800	2FA1D	HAN
END
