# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Pd}
# 	\p{Pd} (and fuzzy permutations)
# 
# Meaning: General Category 'Pd'
#
return <<'END';
002D		
058A		
1806		
2010	2015	
301C		
3030		
30A0		
FE31	FE32	
FE58		
FE63		
FF0D		
END
