# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{LogicalOrderException} (and fuzzy permutations)
# 
# Meaning: Extended property 'Logical_Order_Exception'
#
return <<'END';
0E40	0E44	Logical_Order_Exception
0EC0	0EC4	Logical_Order_Exception
END
