# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCwide}
# 
# Meaning: Compatible with 'wide'
#
return <<'END';
3000		
FF01	FF60	
FFE0	FFE6	
END
