# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCmedial}
# 
# Meaning: Compatible with 'medial'
#
return <<'END';
FB55		
FB59		
FB5D		
FB61		
FB65		
FB69		
FB6D		
FB71		
FB75		
FB79		
FB7D		
FB81		
FB91		
FB95		
FB99		
FB9D		
FBA3		
FBA9		
FBAD		
FBD6		
FBE7		
FBE9		
FBFF		
FCDF	FCF4	
FD34	FD3B	
FE71		
FE77		
FE79		
FE7B		
FE7D		
FE7F		
FE8C		
FE92		
FE98		
FE9C		
FEA0		
FEA4		
FEA8		
FEB4		
FEB8		
FEBC		
FEC0		
FEC4		
FEC8		
FECC		
FED0		
FED4		
FED8		
FEDC		
FEE0		
FEE4		
FEE8		
FEEC		
FEF4		
END
