# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{NoncharacterCodePoint} (and fuzzy permutations)
# 
# Meaning: Extended property 'Noncharacter_Code_Point'
#
return <<'END';
FDD0	FDEF	Noncharacter_Code_Point
FFFE	FFFF	Noncharacter_Code_Point
1FFFE	1FFFF	Noncharacter_Code_Point
2FFFE	2FFFF	Noncharacter_Code_Point
3FFFE	3FFFF	Noncharacter_Code_Point
4FFFE	4FFFF	Noncharacter_Code_Point
5FFFE	5FFFF	Noncharacter_Code_Point
6FFFE	6FFFF	Noncharacter_Code_Point
7FFFE	7FFFF	Noncharacter_Code_Point
8FFFE	8FFFF	Noncharacter_Code_Point
9FFFE	9FFFF	Noncharacter_Code_Point
AFFFE	AFFFF	Noncharacter_Code_Point
BFFFE	BFFFF	Noncharacter_Code_Point
CFFFE	CFFFF	Noncharacter_Code_Point
DFFFE	DFFFF	Noncharacter_Code_Point
EFFFE	EFFFF	Noncharacter_Code_Point
FFFFE	FFFFF	Noncharacter_Code_Point
10FFFE	10FFFF	Noncharacter_Code_Point
END
