# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Pi}
# 	\p{Pi} (and fuzzy permutations)
# 
# Meaning: General Category 'Pi'
#
return <<'END';
00AB		
2018		
201B	201C	
201F		
2039		
END
