# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Hebrew} (and fuzzy permutations)
# 
# Meaning: Script 'HEBREW'
#
return <<'END';
05D0	05EA	HEBREW
05F0	05F2	HEBREW
FB1D		HEBREW
FB1F	FB28	HEBREW
FB2A	FB36	HEBREW
FB38	FB3C	HEBREW
FB3E		HEBREW
FB40	FB41	HEBREW
FB43	FB44	HEBREW
FB46	FB4F	HEBREW
END
