# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Mc}
# 	\p{Mc} (and fuzzy permutations)
# 
# Meaning: General Category 'Mc'
#
return <<'END';
0903		
093E	0940	
0949	094C	
0982	0983	
09BE	09C0	
09C7	09C8	
09CB	09CC	
09D7		
0A03		
0A3E	0A40	
0A83		
0ABE	0AC0	
0AC9		
0ACB	0ACC	
0B02	0B03	
0B3E		
0B40		
0B47	0B48	
0B4B	0B4C	
0B57		
0BBE	0BBF	
0BC1	0BC2	
0BC6	0BC8	
0BCA	0BCC	
0BD7		
0C01	0C03	
0C41	0C44	
0C82	0C83	
0CBE		
0CC0	0CC4	
0CC7	0CC8	
0CCA	0CCB	
0CD5	0CD6	
0D02	0D03	
0D3E	0D40	
0D46	0D48	
0D4A	0D4C	
0D57		
0D82	0D83	
0DCF	0DD1	
0DD8	0DDF	
0DF2	0DF3	
0F3E	0F3F	
0F7F		
102C		
1031		
1038		
1056	1057	
17B6		
17BE	17C5	
17C7	17C8	
1923	1926	
1929	192B	
1930	1931	
1933	1938	
1D165	1D166	
1D16D	1D172	
END
