# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Greek} (and fuzzy permutations)
# 
# Meaning: Script 'GREEK'
#
return <<'END';
00B5		GREEK
037A		GREEK
0386		GREEK
0388	038A	GREEK
038C		GREEK
038E	03A1	GREEK
03A3	03CE	GREEK
03D0	03F5	GREEK
03F7	03FB	GREEK
1D26	1D2A	GREEK
1D5D	1D61	GREEK
1D66	1D6A	GREEK
1F00	1F15	GREEK
1F18	1F1D	GREEK
1F20	1F45	GREEK
1F48	1F4D	GREEK
1F50	1F57	GREEK
1F59		GREEK
1F5B		GREEK
1F5D		GREEK
1F5F	1F7D	GREEK
1F80	1FB4	GREEK
1FB6	1FBC	GREEK
1FBE		GREEK
1FC2	1FC4	GREEK
1FC6	1FCC	GREEK
1FD0	1FD3	GREEK
1FD6	1FDB	GREEK
1FE0	1FEC	GREEK
1FF2	1FF4	GREEK
1FF6	1FFC	GREEK
2126		GREEK
END
