# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiB}
# 
# Meaning: Bi-directional category 'B'
#
return <<'END';
000A		
000D		
001C	001E	
0085		
2029		
END
