# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{IdsBinaryOperator} (and fuzzy permutations)
# 
# Meaning: Extended property 'IDS_Binary_Operator'
#
return <<'END';
2FF0	2FF1	IDS_Binary_Operator
2FF4	2FFB	IDS_Binary_Operator
END
