# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiAN}
# 
# Meaning: Bi-directional category 'AN'
#
return <<'END';
0660	0669	
066B	066C	
END
