# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCsquare}
# 
# Meaning: Compatible with 'square'
#
return <<'END';
3250		
32CC	32CF	
3300	3357	
3371	33DF	
33FF		
END
