# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Inherited} (and fuzzy permutations)
# 
# Meaning: Script 'INHERITED'
#
return <<'END';
0300	034F	INHERITED
0360	036F	INHERITED
0488	0489	INHERITED
0591	05A1	INHERITED
05A3	05B9	INHERITED
05BB	05BD	INHERITED
05BF		INHERITED
05C1	05C2	INHERITED
05C4		INHERITED
064B	0655	INHERITED
0670		INHERITED
06D6	06E4	INHERITED
06E7	06E8	INHERITED
06EA	06ED	INHERITED
180B	180D	INHERITED
20D0	20EA	INHERITED
302A	302F	INHERITED
3099	309A	INHERITED
FB1E		INHERITED
FE00	FE0F	INHERITED
FE20	FE23	INHERITED
1D167	1D169	INHERITED
1D17B	1D182	INHERITED
1D185	1D18B	INHERITED
1D1AA	1D1AD	INHERITED
END
