# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{InYiSyllables} (and fuzzy permutations)
# 
# Meaning: Block 'Yi Syllables'
#
return <<'END';
A000	A48F	Yi Syllables
END
