# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Arabic} (and fuzzy permutations)
# 
# Meaning: Script 'ARABIC'
#
return <<'END';
0621	063A	ARABIC
0641	064A	ARABIC
066E	066F	ARABIC
0671	06D3	ARABIC
06D5		ARABIC
06E5	06E6	ARABIC
06EE	06EF	ARABIC
06FA	06FC	ARABIC
06FF		ARABIC
FB50	FBB1	ARABIC
FBD3	FD3D	ARABIC
FD50	FD8F	ARABIC
FD92	FDC7	ARABIC
FDF0	FDFB	ARABIC
FE70	FE74	ARABIC
FE76	FEFC	ARABIC
END
