# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Limbu} (and fuzzy permutations)
# 
# Meaning: Script 'LIMBU'
#
return <<'END';
1900	191C	LIMBU
1920	192B	LIMBU
1930	193B	LIMBU
1946	194F	LIMBU
END
