# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCsmall}
# 
# Meaning: Compatible with 'small'
#
return <<'END';
FE50	FE52	
FE54	FE66	
FE68	FE6B	
END
