# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Pe}
# 	\p{Pe} (and fuzzy permutations)
# 
# Meaning: General Category 'Pe'
#
return <<'END';
0029		
005D		
007D		
0F3B		
0F3D		
169C		
2046		
207E		
208E		
232A		
23B5		
2769		
276B		
276D		
276F		
2771		
2773		
2775		
27E7		
27E9		
27EB		
2984		
2986		
2988		
298A		
298C		
298E		
2990		
2992		
2994		
2996		
2998		
29D9		
29DB		
29FD		
3009		
300B		
300D		
300F		
3011		
3015		
3017		
3019		
301B		
301E	301F	
FD3F		
FE36		
FE38		
FE3A		
FE3C		
FE3E		
FE40		
FE42		
FE44		
FE48		
FE5A		
FE5C		
FE5E		
FF09		
FF3D		
FF5D		
FF60		
FF63		
END
