# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Cyrillic} (and fuzzy permutations)
# 
# Meaning: Script 'CYRILLIC'
#
return <<'END';
0400	0481	CYRILLIC
0483	0486	CYRILLIC
048A	04CE	CYRILLIC
04D0	04F5	CYRILLIC
04F8	04F9	CYRILLIC
0500	050F	CYRILLIC
1D2B		CYRILLIC
END
