# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Thai} (and fuzzy permutations)
# 
# Meaning: Script 'THAI'
#
return <<'END';
0E01	0E3A	THAI
0E40	0E4E	THAI
0E50	0E59	THAI
END
