# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

##
## Data in this file used by ../utf8_heavy.pl
##

## Mapping from lc(canonical name) to filename in ./lib
%utf8::Canonical = (

 # InAegeanNumbers
   aegeannumbers			    => 'InAegean',

   alphabetic				    => 'Alphabet',

 # InAlphabeticPresentationForms
   alphabeticpresentationforms		    => 'InAlphab',

   arabic				    => 'Arabic',

 # InArabicPresentationFormsA
   arabicpresentationformsa		    => 'InArabi3',

 # InArabicPresentationFormsB
   arabicpresentationformsb		    => 'InArabi2',

   armenian				    => 'Armenian',

 # InArrows
   arrows				    => 'InArrows',

 # AsciiHexDigit
   asciihexdigit			    => 'AsciiHex',

 # InBasicLatin
   basiclatin				    => 'InBasicL',

   bengali				    => 'Bengali',

 # BidiControl
   bidicontrol				    => 'BidiCont',

 # InBlockElements
   blockelements			    => 'InBlockE',

   bopomofo				    => 'Bopomofo',

 # InBopomofoExtended
   bopomofoextended			    => 'InBopom2',

 # InBoxDrawing
   boxdrawing				    => 'InBoxDra',

   braille				    => 'Braille',

 # InBraillePatterns
   braillepatterns			    => 'InBraill',

   buhid				    => 'Buhid',

 # InByzantineMusicalSymbols
   byzantinemusicalsymbols		    => 'InByzant',

 # CanadianAboriginal
   canadianaboriginal			    => 'Canadian',

   cherokee				    => 'Cherokee',

 # InCjkCompatibility
   cjkcompatibility			    => 'InCjkCom',

 # InCjkCompatibilityForms
   cjkcompatibilityforms		    => 'InCjkCo2',

 # InCjkCompatibilityIdeographs
   cjkcompatibilityideographs		    => 'InCjkCo3',

 # InCjkCompatibilityIdeographsSupplement
   cjkcompatibilityideographssupplement     => 'InCjkCo4',

 # InCjkRadicalsSupplement
   cjkradicalssupplement		    => 'InCjkRad',

 # InCjkSymbolsAndPunctuation
   cjksymbolsandpunctuation		    => 'InCjkSym',

 # InCjkUnifiedIdeographs
   cjkunifiedideographs 		    => 'InCjkUni',

 # InCjkUnifiedIdeographsExtensionA
   cjkunifiedideographsextensiona	    => 'InCjkUn3',

 # InCjkUnifiedIdeographsExtensionB
   cjkunifiedideographsextensionb	    => 'InCjkUn2',

 # ClosePunctuation
   closepunctuation			    => 'Pe',

 # InCombiningDiacriticalMarks
   combiningdiacriticalmarks		    => 'InCombi2',

 # InCombiningDiacriticalMarksForSymbols
   combiningdiacriticalmarksforsymbols	    => 'InCombi3',

 # InCombiningHalfMarks
   combininghalfmarks			    => 'InCombin',

   common				    => 'Common',

 # ConnectorPunctuation
   connectorpunctuation 		    => 'Pc',

   control				    => 'Cc',

 # InControlPictures
   controlpictures			    => 'InContro',

 # CurrencySymbol
   currencysymbol			    => 'Sc',

 # InCurrencySymbols
   currencysymbols			    => 'InCurren',

   cypriot				    => 'Cypriot',

 # InCypriotSyllabary
   cypriotsyllabary			    => 'InCyprio',

   cyrillic				    => 'Cyrillic',

 # InCyrillicSupplementary
   cyrillicsupplementary		    => 'InCyril2',

   dash 				    => 'Dash',

 # DashPunctuation
   dashpunctuation			    => 'Pd',

 # DecimalNumber
   decimalnumber			    => 'Nd',

   deprecated				    => 'Deprecat',
   deseret				    => 'Deseret',
   devanagari				    => 'Devanaga',
   diacritic				    => 'Diacriti',

 # InDingbats
   dingbats				    => 'InDingba',

 # InEnclosedAlphanumerics
   enclosedalphanumerics		    => 'InEnclos',

 # InEnclosedCjkLettersAndMonths
   enclosedcjklettersandmonths		    => 'InEnclo2',

 # EnclosingMark
   enclosingmark			    => 'Me',

   ethiopic				    => 'Ethiopic',
   extender				    => 'Extender',

 # FinalPunctuation
   finalpunctuation			    => 'Pf',

   format				    => 'Cf',

 # InGeneralPunctuation
   generalpunctuation			    => 'InGenera',

 # InGeometricShapes
   geometricshapes			    => 'InGeomet',

   georgian				    => 'Georgian',
   gothic				    => 'Gothic',

 # GraphemeLink
   graphemelink 			    => 'Grapheme',

   greek				    => 'Greek',

 # InGreekAndCoptic
   greekandcoptic			    => 'InGreekA',

 # InGreekExtended
   greekextended			    => 'InGreekE',

   gujarati				    => 'Gujarati',
   gurmukhi				    => 'Gurmukhi',

 # InHalfwidthAndFullwidthForms
   halfwidthandfullwidthforms		    => 'InHalfwi',

   han					    => 'Han',
   hangul				    => 'Hangul',

 # InHangulCompatibilityJamo
   hangulcompatibilityjamo		    => 'InHangu3',

 # InHangulJamo
   hanguljamo				    => 'InHangul',

 # InHangulSyllables
   hangulsyllables			    => 'InHangu2',

   hanunoo				    => 'Hanunoo',
   hebrew				    => 'Hebrew',

 # HexDigit
   hexdigit				    => 'HexDigit',

 # InHighPrivateUseSurrogates
   highprivateusesurrogates		    => 'InHighPr',

 # InHighSurrogates
   highsurrogates			    => 'InHighSu',

   hiragana				    => 'Hiragana',
   hyphen				    => 'Hyphen',

 # IdContinue
   idcontinue				    => 'IdContin',

   ideographic				    => 'Ideograp',

 # InIdeographicDescriptionCharacters
   ideographicdescriptioncharacters	    => 'InIdeogr',

 # IdsBinaryOperator
   idsbinaryoperator			    => 'IdsBinar',

 # IdStart
   idstart				    => 'IdStart',

 # IdsTrinaryOperator
   idstrinaryoperator			    => 'IdsTrina',

 # InAegeanNumbers
   inaegeannumbers			    => 'InAegean',

 # InAlphabeticPresentationForms
   inalphabeticpresentationforms	    => 'InAlphab',

 # InArabic
   inarabic				    => 'InArabic',

 # InArabicPresentationFormsA
   inarabicpresentationformsa		    => 'InArabi3',

 # InArabicPresentationFormsB
   inarabicpresentationformsb		    => 'InArabi2',

 # InArmenian
   inarmenian				    => 'InArmeni',

 # InArrows
   inarrows				    => 'InArrows',

 # InBasicLatin
   inbasiclatin 			    => 'InBasicL',

 # InBengali
   inbengali				    => 'InBengal',

 # InBlockElements
   inblockelements			    => 'InBlockE',

 # InBopomofo
   inbopomofo				    => 'InBopomo',

 # InBopomofoExtended
   inbopomofoextended			    => 'InBopom2',

 # InBoxDrawing
   inboxdrawing 			    => 'InBoxDra',

 # InBraillePatterns
   inbraillepatterns			    => 'InBraill',

 # InBuhid
   inbuhid				    => 'InBuhid',

 # InByzantineMusicalSymbols
   inbyzantinemusicalsymbols		    => 'InByzant',

 # InCherokee
   incherokee				    => 'InCherok',

 # InCjkCompatibility
   incjkcompatibility			    => 'InCjkCom',

 # InCjkCompatibilityForms
   incjkcompatibilityforms		    => 'InCjkCo2',

 # InCjkCompatibilityIdeographs
   incjkcompatibilityideographs 	    => 'InCjkCo3',

 # InCjkCompatibilityIdeographsSupplement
   incjkcompatibilityideographssupplement   => 'InCjkCo4',

 # InCjkRadicalsSupplement
   incjkradicalssupplement		    => 'InCjkRad',

 # InCjkSymbolsAndPunctuation
   incjksymbolsandpunctuation		    => 'InCjkSym',

 # InCjkUnifiedIdeographs
   incjkunifiedideographs		    => 'InCjkUni',

 # InCjkUnifiedIdeographsExtensionA
   incjkunifiedideographsextensiona	    => 'InCjkUn3',

 # InCjkUnifiedIdeographsExtensionB
   incjkunifiedideographsextensionb	    => 'InCjkUn2',

 # InCombiningDiacriticalMarks
   incombiningdiacriticalmarks		    => 'InCombi2',

 # InCombiningDiacriticalMarksForSymbols
   incombiningdiacriticalmarksforsymbols    => 'InCombi3',

 # InCombiningHalfMarks
   incombininghalfmarks 		    => 'InCombin',

 # InControlPictures
   incontrolpictures			    => 'InContro',

 # InCurrencySymbols
   incurrencysymbols			    => 'InCurren',

 # InCypriotSyllabary
   incypriotsyllabary			    => 'InCyprio',

 # InCyrillic
   incyrillic				    => 'InCyrill',

 # InCyrillicSupplementary
   incyrillicsupplementary		    => 'InCyril2',

 # InDeseret
   indeseret				    => 'InDesere',

 # InDevanagari
   indevanagari 			    => 'InDevana',

 # InDingbats
   indingbats				    => 'InDingba',

 # InEnclosedAlphanumerics
   inenclosedalphanumerics		    => 'InEnclos',

 # InEnclosedCjkLettersAndMonths
   inenclosedcjklettersandmonths	    => 'InEnclo2',

 # InEthiopic
   inethiopic				    => 'InEthiop',

 # InGeneralPunctuation
   ingeneralpunctuation 		    => 'InGenera',

 # InGeometricShapes
   ingeometricshapes			    => 'InGeomet',

 # InGeorgian
   ingeorgian				    => 'InGeorgi',

 # InGothic
   ingothic				    => 'InGothic',

 # InGreekAndCoptic
   ingreekandcoptic			    => 'InGreekA',

 # InGreekExtended
   ingreekextended			    => 'InGreekE',

 # InGujarati
   ingujarati				    => 'InGujara',

 # InGurmukhi
   ingurmukhi				    => 'InGurmuk',

 # InHalfwidthAndFullwidthForms
   inhalfwidthandfullwidthforms 	    => 'InHalfwi',

 # InHangulCompatibilityJamo
   inhangulcompatibilityjamo		    => 'InHangu3',

 # InHangulJamo
   inhanguljamo 			    => 'InHangul',

 # InHangulSyllables
   inhangulsyllables			    => 'InHangu2',

 # InHanunoo
   inhanunoo				    => 'InHanuno',

 # InHebrew
   inhebrew				    => 'InHebrew',

   inherited				    => 'Inherite',

 # InHighPrivateUseSurrogates
   inhighprivateusesurrogates		    => 'InHighPr',

 # InHighSurrogates
   inhighsurrogates			    => 'InHighSu',

 # InHiragana
   inhiragana				    => 'InHiraga',

 # InIdeographicDescriptionCharacters
   inideographicdescriptioncharacters	    => 'InIdeogr',

 # InIpaExtensions
   inipaextensions			    => 'InIpaExt',

 # InitialPunctuation
   initialpunctuation			    => 'Pi',

 # InKanbun
   inkanbun				    => 'InKanbun',

 # InKangxiRadicals
   inkangxiradicals			    => 'InKangxi',

 # InKannada
   inkannada				    => 'InKannad',

 # InKatakana
   inkatakana				    => 'InKataka',

 # InKatakanaPhoneticExtensions
   inkatakanaphoneticextensions 	    => 'InKatak2',

 # InKhmer
   inkhmer				    => 'InKhmer',

 # InKhmerSymbols
   inkhmersymbols			    => 'InKhmerS',

 # InLao
   inlao				    => 'InLao',

 # InLatin1Supplement
   inlatin1supplement			    => 'InLatin1',

 # InLatinExtendedA
   inlatinextendeda			    => 'InLatin2',

 # InLatinExtendedAdditional
   inlatinextendedadditional		    => 'InLatin3',

 # InLatinExtendedB
   inlatinextendedb			    => 'InLatinE',

 # InLetterlikeSymbols
   inletterlikesymbols			    => 'InLetter',

 # InLimbu
   inlimbu				    => 'InLimbu',

 # InLinearBIdeograms
   inlinearbideograms			    => 'InLinear',

 # InLinearBSyllabary
   inlinearbsyllabary			    => 'InLinea2',

 # InLowSurrogates
   inlowsurrogates			    => 'InLowSur',

 # InMalayalam
   inmalayalam				    => 'InMalaya',

 # InMathematicalAlphanumericSymbols
   inmathematicalalphanumericsymbols	    => 'InMathe2',

 # InMathematicalOperators
   inmathematicaloperators		    => 'InMathem',

 # InMiscellaneousMathematicalSymbolsA
   inmiscellaneousmathematicalsymbolsa	    => 'InMisce4',

 # InMiscellaneousMathematicalSymbolsB
   inmiscellaneousmathematicalsymbolsb	    => 'InMisce5',

 # InMiscellaneousSymbols
   inmiscellaneoussymbols		    => 'InMiscel',

 # InMiscellaneousSymbolsAndArrows
   inmiscellaneoussymbolsandarrows	    => 'InMisce3',

 # InMiscellaneousTechnical
   inmiscellaneoustechnical		    => 'InMisce2',

 # InMongolian
   inmongolian				    => 'InMongol',

 # InMusicalSymbols
   inmusicalsymbols			    => 'InMusica',

 # InMyanmar
   inmyanmar				    => 'InMyanma',

 # InNumberForms
   innumberforms			    => 'InNumber',

 # InOgham
   inogham				    => 'InOgham',

 # InOldItalic
   inolditalic				    => 'InOldIta',

 # InOpticalCharacterRecognition
   inopticalcharacterrecognition	    => 'InOptica',

 # InOriya
   inoriya				    => 'InOriya',

 # InOsmanya
   inosmanya				    => 'InOsmany',

 # InPhoneticExtensions
   inphoneticextensions 		    => 'InPhonet',

 # InPrivateUseArea
   inprivateusearea			    => 'InPrivat',

 # InRunic
   inrunic				    => 'InRunic',

 # InShavian
   inshavian				    => 'InShavia',

 # InSinhala
   insinhala				    => 'InSinhal',

 # InSmallFormVariants
   insmallformvariants			    => 'InSmallF',

 # InSpacingModifierLetters
   inspacingmodifierletters		    => 'InSpacin',

 # InSpecials
   inspecials				    => 'InSpecia',

 # InSuperscriptsAndSubscripts
   insuperscriptsandsubscripts		    => 'InSupers',

 # InSupplementalArrowsA
   insupplementalarrowsa		    => 'InSuppl2',

 # InSupplementalArrowsB
   insupplementalarrowsb		    => 'InSupple',

 # InSupplementalMathematicalOperators
   insupplementalmathematicaloperators	    => 'InSuppl5',

 # InSupplementaryPrivateUseAreaA
   insupplementaryprivateuseareaa	    => 'InSuppl3',

 # InSupplementaryPrivateUseAreaB
   insupplementaryprivateuseareab	    => 'InSuppl4',

 # InSyriac
   insyriac				    => 'InSyriac',

 # InTagalog
   intagalog				    => 'InTagalo',

 # InTagbanwa
   intagbanwa				    => 'InTagban',

 # InTags
   intags				    => 'InTags',

 # InTaiLe
   intaile				    => 'InTaiLe',

 # InTaiXuanJingSymbols
   intaixuanjingsymbols 		    => 'InTaiXua',

 # InTamil
   intamil				    => 'InTamil',

 # InTelugu
   intelugu				    => 'InTelugu',

 # InThaana
   inthaana				    => 'InThaana',

 # InThai
   inthai				    => 'InThai',

 # InTibetan
   intibetan				    => 'InTibeta',

 # InUgaritic
   inugaritic				    => 'InUgarit',

 # InUnifiedCanadianAboriginalSyllabics
   inunifiedcanadianaboriginalsyllabics     => 'InUnifie',

 # InVariationSelectors
   invariationselectors 		    => 'InVariat',

 # InVariationSelectorsSupplement
   invariationselectorssupplement	    => 'InVaria2',

 # InYijingHexagramSymbols
   inyijinghexagramsymbols		    => 'InYijing',

 # InYiRadicals
   inyiradicals 			    => 'InYiRadi',

 # InYiSyllables
   inyisyllables			    => 'InYiSyll',

 # InIpaExtensions
   ipaextensions			    => 'InIpaExt',

 # JoinControl
   joincontrol				    => 'JoinCont',

 # InKanbun
   kanbun				    => 'InKanbun',

 # InKangxiRadicals
   kangxiradicals			    => 'InKangxi',

   kannada				    => 'Kannada',
   katakana				    => 'Katakana',

 # InKatakanaPhoneticExtensions
   katakanaphoneticextensions		    => 'InKatak2',

   khmer				    => 'Khmer',

 # InKhmerSymbols
   khmersymbols 			    => 'InKhmerS',

   lao					    => 'Lao',
   latin				    => 'Latin',

 # InLatin1Supplement
   latin1supplement			    => 'InLatin1',

 # InLatinExtendedA
   latinextendeda			    => 'InLatin2',

 # InLatinExtendedAdditional
   latinextendedadditional		    => 'InLatin3',

 # InLatinExtendedB
   latinextendedb			    => 'InLatinE',

   letter				    => 'L',

 # InLetterlikeSymbols
   letterlikesymbols			    => 'InLetter',

 # LetterNumber
   letternumber 			    => 'Nl',

   limbu				    => 'Limbu',

 # LinearB
   linearb				    => 'LinearB',

 # InLinearBIdeograms
   linearbideograms			    => 'InLinear',

 # InLinearBSyllabary
   linearbsyllabary			    => 'InLinea2',

 # LineSeparator
   lineseparator			    => 'Zl',

 # LogicalOrderException
   logicalorderexception		    => 'LogicalO',

   lowercase				    => 'Lowercas',

 # LowercaseLetter
   lowercaseletter			    => 'Ll',

 # InLowSurrogates
   lowsurrogates			    => 'InLowSur',

   malayalam				    => 'Malayala',
   mark 				    => 'M',
   math 				    => 'Math',

 # InMathematicalAlphanumericSymbols
   mathematicalalphanumericsymbols	    => 'InMathe2',

 # InMathematicalOperators
   mathematicaloperators		    => 'InMathem',

 # MathSymbol
   mathsymbol				    => 'Sm',

 # InMiscellaneousMathematicalSymbolsA
   miscellaneousmathematicalsymbolsa	    => 'InMisce4',

 # InMiscellaneousMathematicalSymbolsB
   miscellaneousmathematicalsymbolsb	    => 'InMisce5',

 # InMiscellaneousSymbols
   miscellaneoussymbols 		    => 'InMiscel',

 # InMiscellaneousSymbolsAndArrows
   miscellaneoussymbolsandarrows	    => 'InMisce3',

 # InMiscellaneousTechnical
   miscellaneoustechnical		    => 'InMisce2',

 # ModifierLetter
   modifierletter			    => 'Lm',

 # ModifierSymbol
   modifiersymbol			    => 'Sk',

   mongolian				    => 'Mongolia',

 # InMusicalSymbols
   musicalsymbols			    => 'InMusica',

   myanmar				    => 'Myanmar',

 # NoncharacterCodePoint
   noncharactercodepoint		    => 'Nonchara',

 # NonSpacingMark
   nonspacingmark			    => 'Mn',

   number				    => 'N',

 # InNumberForms
   numberforms				    => 'InNumber',

   ogham				    => 'Ogham',

 # OldItalic
   olditalic				    => 'OldItali',

 # OpenPunctuation
   openpunctuation			    => 'Ps',

 # InOpticalCharacterRecognition
   opticalcharacterrecognition		    => 'InOptica',

   oriya				    => 'Oriya',
   osmanya				    => 'Osmanya',
   other				    => 'C',

 # OtherAlphabetic
   otheralphabetic			    => 'OtherAlp',

 # OtherDefaultIgnorableCodePoint
   otherdefaultignorablecodepoint	    => 'OtherDef',

 # OtherGraphemeExtend
   othergraphemeextend			    => 'OtherGra',

 # OtherIdStart
   otheridstart 			    => 'OtherIdS',

 # OtherLetter
   otherletter				    => 'Lo',

 # OtherLowercase
   otherlowercase			    => 'OtherLow',

 # OtherMath
   othermath				    => 'OtherMat',

 # OtherNumber
   othernumber				    => 'No',

 # OtherPunctuation
   otherpunctuation			    => 'Po',

 # OtherSymbol
   othersymbol				    => 'So',

 # OtherUppercase
   otheruppercase			    => 'OtherUpp',

 # ParagraphSeparator
   paragraphseparator			    => 'Zp',

 # InPhoneticExtensions
   phoneticextensions			    => 'InPhonet',

 # PrivateUse
   privateuse				    => 'Co',

 # InPrivateUseArea
   privateusearea			    => 'InPrivat',

   punctuation				    => 'P',

 # QuotationMark
   quotationmark			    => 'Quotatio',

   radical				    => 'Radical',
   runic				    => 'Runic',
   separator				    => 'Z',
   shavian				    => 'Shavian',
   sinhala				    => 'Sinhala',

 # InSmallFormVariants
   smallformvariants			    => 'InSmallF',

 # SoftDotted
   softdotted				    => 'SoftDott',

 # SpaceSeparator
   spaceseparator			    => 'Zs',

 # SpacingMark
   spacingmark				    => 'Mc',

 # InSpacingModifierLetters
   spacingmodifierletters		    => 'InSpacin',

 # InSpecials
   specials				    => 'InSpecia',

 # InSuperscriptsAndSubscripts
   superscriptsandsubscripts		    => 'InSupers',

 # InSupplementalArrowsA
   supplementalarrowsa			    => 'InSuppl2',

 # InSupplementalArrowsB
   supplementalarrowsb			    => 'InSupple',

 # InSupplementalMathematicalOperators
   supplementalmathematicaloperators	    => 'InSuppl5',

 # InSupplementaryPrivateUseAreaA
   supplementaryprivateuseareaa 	    => 'InSuppl3',

 # InSupplementaryPrivateUseAreaB
   supplementaryprivateuseareab 	    => 'InSuppl4',

   surrogate				    => 'Cs',
   symbol				    => 'S',
   syriac				    => 'Syriac',
   tagalog				    => 'Tagalog',
   tagbanwa				    => 'Tagbanwa',

 # InTags
   tags 				    => 'InTags',

 # TaiLe
   taile				    => 'TaiLe',

 # InTaiXuanJingSymbols
   taixuanjingsymbols			    => 'InTaiXua',

   tamil				    => 'Tamil',
   telugu				    => 'Telugu',

 # TerminalPunctuation
   terminalpunctuation			    => 'Terminal',

   thaana				    => 'Thaana',
   thai 				    => 'Thai',
   tibetan				    => 'Tibetan',

 # TitlecaseLetter
   titlecaseletter			    => 'Lt',

   ugaritic				    => 'Ugaritic',
   unassigned				    => 'Cn',

 # InUnifiedCanadianAboriginalSyllabics
   unifiedcanadianaboriginalsyllabics	    => 'InUnifie',

 # UnifiedIdeograph
   unifiedideograph			    => 'UnifiedI',

   uppercase				    => 'Uppercas',

 # UppercaseLetter
   uppercaseletter			    => 'Lu',

 # InVariationSelectors
   variationselectors			    => 'InVariat',

 # InVariationSelectorsSupplement
   variationselectorssupplement 	    => 'InVaria2',

 # WhiteSpace
   whitespace				    => 'WhiteSpa',

   yi					    => 'Yi',

 # InYijingHexagramSymbols
   yijinghexagramsymbols		    => 'InYijing',

 # InYiRadicals
   yiradicals				    => 'InYiRadi',

 # InYiSyllables
   yisyllables				    => 'InYiSyll',

);
1
