# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{OtherIdStart} (and fuzzy permutations)
# 
# Meaning: Extended property 'Other_ID_Start'
#
return <<'END';
2118		Other_ID_Start
212E		Other_ID_Start
309B	309C	Other_ID_Start
END
