# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{SoftDotted} (and fuzzy permutations)
# 
# Meaning: Extended property 'Soft_Dotted'
#
return <<'END';
0069	006A	Soft_Dotted
012F		Soft_Dotted
0268		Soft_Dotted
0456		Soft_Dotted
0458		Soft_Dotted
1D62		Soft_Dotted
1E2D		Soft_Dotted
1ECB		Soft_Dotted
2071		Soft_Dotted
END
