# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiET}
# 
# Meaning: Bi-directional category 'ET'
#
return <<'END';
0023	0025	
002B		
002D		
00A2	00A5	
00B0	00B1	
066A		
09F2	09F3	
0AF1		
0BF9		
0E3F		
17DB		
2030	2034	
207A	207B	
208A	208B	
20A0	20B1	
212E		
2212	2213	
FB29		
FE5F		
FE62	FE63	
FE69	FE6A	
FF03	FF05	
FF0B		
FF0D		
FFE0	FFE1	
FFE5	FFE6	
END
