# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Mongolian} (and fuzzy permutations)
# 
# Meaning: Script 'MONGOLIAN'
#
return <<'END';
1810	1819	MONGOLIAN
1820	1877	MONGOLIAN
1880	18A9	MONGOLIAN
END
