# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Kannada} (and fuzzy permutations)
# 
# Meaning: Script 'KANNADA'
#
return <<'END';
0C82	0C83	KANNADA
0C85	0C8C	KANNADA
0C8E	0C90	KANNADA
0C92	0CA8	KANNADA
0CAA	0CB3	KANNADA
0CB5	0CB9	KANNADA
0CBD	0CC4	KANNADA
0CC6	0CC8	KANNADA
0CCA	0CCD	KANNADA
0CD5	0CD6	KANNADA
0CDE		KANNADA
0CE0	0CE1	KANNADA
0CE6	0CEF	KANNADA
END
