# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Myanmar} (and fuzzy permutations)
# 
# Meaning: Script 'MYANMAR'
#
return <<'END';
1000	1021	MYANMAR
1023	1027	MYANMAR
1029	102A	MYANMAR
102C	1032	MYANMAR
1036	1039	MYANMAR
1040	1049	MYANMAR
1050	1059	MYANMAR
END
