# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiAL}
# 
# Meaning: Bi-directional category 'AL'
#
return <<'END';
0600	0603	
060D		
061B		
061F		
0621	063A	
0640	064A	
066D	066F	
0671	06D5	
06DD		
06E5	06E6	
06EE	06EF	
06FA	070D	
0710		
0712	072F	
074D	074F	
0780	07A5	
07B1		
FB50	FBB1	
FBD3	FD3D	
FD50	FD8F	
FD92	FDC7	
FDF0	FDFC	
FE70	FE74	
FE76	FEFC	
END
