# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Hyphen} (and fuzzy permutations)
# 
# Meaning: Extended property 'Hyphen'
#
return <<'END';
002D		Hyphen
00AD		Hyphen
058A		Hyphen
1806		Hyphen
2010	2011	Hyphen
30FB		Hyphen
FE63		Hyphen
FF0D		Hyphen
FF65		Hyphen
END
