# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCfinal}
# 
# Meaning: Compatible with 'final'
#
return <<'END';
FB51		
FB53		
FB57		
FB5B		
FB5F		
FB63		
FB67		
FB6B		
FB6F		
FB73		
FB77		
FB7B		
FB7F		
FB83		
FB85		
FB87		
FB89		
FB8B		
FB8D		
FB8F		
FB93		
FB97		
FB9B		
FB9F		
FBA1		
FBA5		
FBA7		
FBAB		
FBAF		
FBB1		
FBD4		
FBD8		
FBDA		
FBDC		
FBDF		
FBE1		
FBE3		
FBE5		
FBEB		
FBED		
FBEF		
FBF1		
FBF3		
FBF5		
FBF7		
FBFA		
FBFD		
FC64	FC96	
FD11	FD2C	
FD3C		
FD51		
FD58		
FD5A	FD5B	
FD5E	FD5F	
FD62		
FD64		
FD66	FD67	
FD69	FD6A	
FD6C		
FD6E	FD6F	
FD71		
FD74	FD76	
FD78	FD7C	
FD7E	FD82	
FD84	FD85	
FD87		
FD8B		
FD96	FD97	
FD99	FD9C	
FD9E	FDB3	
FDB6	FDB7	
FDB9		
FDBB	FDC2	
FDC6	FDC7	
FE82		
FE84		
FE86		
FE88		
FE8A		
FE8E		
FE90		
FE94		
FE96		
FE9A		
FE9E		
FEA2		
FEA6		
FEAA		
FEAC		
FEAE		
FEB0		
FEB2		
FEB6		
FEBA		
FEBE		
FEC2		
FEC6		
FECA		
FECE		
FED2		
FED6		
FEDA		
FEDE		
FEE2		
FEE6		
FEEA		
FEEE		
FEF0		
FEF2		
FEF6		
FEF8		
FEFA		
FEFC		
END
