# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Tibetan} (and fuzzy permutations)
# 
# Meaning: Script 'TIBETAN'
#
return <<'END';
0F00		TIBETAN
0F18	0F19	TIBETAN
0F20	0F33	TIBETAN
0F35		TIBETAN
0F37		TIBETAN
0F39		TIBETAN
0F40	0F47	TIBETAN
0F49	0F6A	TIBETAN
0F71	0F84	TIBETAN
0F86	0F8B	TIBETAN
0F90	0F97	TIBETAN
0F99	0FBC	TIBETAN
0FC6		TIBETAN
END
